/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "malem-aligned-window.h"

enum {
	PROP_0,
	STATUS_ICON
};

G_DEFINE_TYPE (MalemAlignedWindow, malem_aligned_window, GTK_TYPE_WINDOW)

static void
malem_aligned_window_init (MalemAlignedWindow *window)
{
	GtkWindow *gtk_window = GTK_WINDOW (window);

	gtk_window_set_decorated (gtk_window, FALSE);

	/* Skip the taskbar, and the pager, stick and stay on top */	
	gtk_window_stick (gtk_window);
	gtk_window_set_keep_above (gtk_window, TRUE);
	gtk_window_set_skip_pager_hint (gtk_window, TRUE);
	gtk_window_set_skip_taskbar_hint (gtk_window, TRUE);
	gtk_window_set_type_hint (gtk_window, GDK_WINDOW_TYPE_HINT_MENU);
	
	window->frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (window->frame), GTK_SHADOW_OUT);
	(* GTK_CONTAINER_CLASS (malem_aligned_window_parent_class)->add) 
					(GTK_CONTAINER (window), window->frame);
	gtk_widget_show (window->frame);

	g_signal_connect (window, "realize", 
			  G_CALLBACK (malem_aligned_window_update_position),
			  NULL);
}

static void
malem_aligned_window_get_property (GObject    *object,
				   guint       prop_id,
				   GValue     *value,
				   GParamSpec *pspec)
{
	MalemAlignedWindow *window = MALEM_ALIGNED_WINDOW (object);

	switch (prop_id)
	{
		case STATUS_ICON:
			g_value_set_object (value, window->icon);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
set_icon (MalemAlignedWindow *window,
	  GtkStatusIcon      *icon)
{
	GtkStatusIcon **w;

	g_return_if_fail (GTK_IS_STATUS_ICON (icon));

	w = &window->icon;

	if (*w != NULL) {
		g_object_unref (*w);
		g_object_remove_weak_pointer (G_OBJECT (*w), (gpointer *) w);
	}
	*w = icon;
	g_object_ref (*w);
	g_object_add_weak_pointer (G_OBJECT (*w), (gpointer *) w);
	gtk_widget_realize (GTK_WIDGET (window));
}

static void
malem_aligned_window_set_property (GObject      *object,
				   guint         prop_id,
				   const GValue *value,
				   GParamSpec   *pspec)
{
	MalemAlignedWindow *window = MALEM_ALIGNED_WINDOW (object);

	switch (prop_id)
	{
		case STATUS_ICON:
			set_icon (window, g_value_get_object (value));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
malem_aligned_window_finalize (GObject *object)
{
	MalemAlignedWindow *window = MALEM_ALIGNED_WINDOW (object);

	if (window->icon != NULL)
		g_object_unref (window->icon);

	/* chainup */
	G_OBJECT_CLASS (malem_aligned_window_parent_class)->finalize (object);
}

static void
malem_aligned_window_add (GtkContainer *container,
			  GtkWidget    *child)
{
	MalemAlignedWindow *window = MALEM_ALIGNED_WINDOW (container);

	gtk_container_add (GTK_CONTAINER (window->frame), child);
}

static void
malem_aligned_window_remove (GtkContainer *container,
			     GtkWidget    *child)
{
	MalemAlignedWindow *window = MALEM_ALIGNED_WINDOW (container);

	if (gtk_bin_get_child (GTK_BIN (window->frame)) == child)
		gtk_container_remove (GTK_CONTAINER (window->frame), child);
	else
		GTK_CONTAINER_CLASS (malem_aligned_window_parent_class)->remove (container, child);
}

static void
malem_aligned_window_class_init (MalemAlignedWindowClass *klass)
{
	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);

	object_class->set_property = malem_aligned_window_set_property;
	object_class->get_property = malem_aligned_window_get_property;
	object_class->finalize = malem_aligned_window_finalize;
	
	container_class->add = malem_aligned_window_add;
	container_class->remove = malem_aligned_window_remove;
	
	g_object_class_install_property (G_OBJECT_CLASS (klass), STATUS_ICON,
					 g_param_spec_object ("status-icon",
					 		      "Status Icon",
					 		      "The status icon the window is aligned with",
					 		      GTK_TYPE_STATUS_ICON,
					 		      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
}

GtkWidget *
malem_aligned_window_new (GtkStatusIcon *icon)
{
	MalemAlignedWindow *window;
	
	window = MALEM_ALIGNED_WINDOW (g_object_new (MALEM_TYPE_ALIGNED_WINDOW,
						     "status-icon", icon,
						      NULL));

	return GTK_WIDGET (window);
}

#if 0
void
malem_aligned_window_update_position (MalemAlignedWindow *window)
{
	GtkRequisition requisition;
	GdkScreen *screen;
	GdkRectangle area;
	GtkOrientation orientation;
	gint x, y;

	g_return_if_fail (window->icon != NULL);

	if (!GTK_WIDGET_REALIZED (window)) {
		gtk_widget_realize (GTK_WIDGET (window));
		return;
	}

	gtk_status_icon_get_geometry (window->icon, &screen, &area, &orientation);

	gtk_widget_size_request (GTK_WIDGET (window), &requisition);

	x = area.x;
	y = area.y;

	if (y > gdk_screen_get_height (screen) / 2)
		y -= requisition.height + 1;
	else
		y += area.height - 1;

	gtk_window_move (GTK_WINDOW (window), x, y);
}
#endif

/* adapted from gtkstatusicon.c */
void
malem_aligned_window_update_position (MalemAlignedWindow *window)
{
	GdkScreen *screen;
	GdkRectangle area;
	GtkOrientation orientation;
	GtkTextDirection direction;
	GtkRequisition window_req;
	gint monitor_num;
	GdkRectangle monitor;
	gint height, width, xoffset, yoffset, x, y;

	g_return_if_fail (window->icon != NULL);

	gtk_status_icon_get_geometry (window->icon, &screen, &area, &orientation);

	direction = gtk_widget_get_direction (GTK_WIDGET (window));

	gtk_window_set_screen (GTK_WINDOW (window), screen);
	
	monitor_num = gdk_screen_get_monitor_at_point (screen, area.x, area.y);
	if (monitor_num < 0)
		monitor_num = 0;
	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);

	x = area.x;
	y = area.y;
	gtk_widget_size_request (GTK_WIDGET (window), &window_req);

	if (orientation == GTK_ORIENTATION_VERTICAL) {
		width = 0;
		height = area.height;
		xoffset = area.width;
		yoffset = 0;
	} else {
		width = area.width;
		height = 0;
		xoffset = 0;
		yoffset = area.height;
	}

	if (direction == GTK_TEXT_DIR_RTL) {
		if ((x - (window_req.width - width)) >= monitor.x)
			x -= window_req.width - width;
		else if ((x + xoffset + window_req.width) < (monitor.x + monitor.width))
			x += xoffset;
		else if ((monitor.x + monitor.width - (x + xoffset)) < x)
			x -= window_req.width - width;
		else
			x += xoffset;
	} else {
		if ((x + xoffset + window_req.width) < (monitor.x + monitor.width))
			x += xoffset;
		else if ((x - (window_req.width - width)) >= monitor.x)
			x -= window_req.width - width;
		else if ((monitor.x + monitor.width - (x + xoffset)) > x)
			x += xoffset;
		else 
			x -= window_req.width - width;
	}
	
	if ((y + yoffset + window_req.height) < (monitor.y + monitor.height))
		y += yoffset;
	else if ((y - (window_req.height - height)) >= monitor.y)
		y -= window_req.height - height;
	else if (monitor.y + monitor.height - (y + yoffset) > y)
		y += yoffset;
	else 
		y -= window_req.height - height;

	gtk_window_move (GTK_WINDOW (window), x, y);
}
