/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __MALEM_DICTIONARY_H__
#define __MALEM_DICTIONARY_H__

#include <glib-object.h>

G_BEGIN_DECLS

#define MALEM_TYPE_DICTIONARY			(malem_dictionary_get_type ())
#define MALEM_DICTIONARY(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), MALEM_TYPE_DICTIONARY, MalemDictionary))
#define MALEM_DICTIONARY_CLASS(obj)		(G_TYPE_CHECK_CLASS_CAST ((obj), MALEM_TYPE_DICTIONARY, MalemDictionaryIface))
#define MALEM_IS_DICTIONARY(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), MALEM_TYPE_DICTIONARY))
#define MALEM_DICTIONARY_GET_IFACE(obj)	(G_TYPE_INSTANCE_GET_INTERFACE ((obj), MALEM_TYPE_DICTIONARY, MalemDictionaryIface))

typedef struct _MalemDictionary	MalemDictionary; /* dummy */
typedef struct _MalemDictionaryIface	MalemDictionaryIface;

struct _MalemDictionaryIface {
	GTypeInterface interface;
	
	/* VTable */
	gboolean	  (* get_value)		(MalemDictionary *dictionary,
						 const gchar     *key,
						 GValue          *value);
	void		  (* set_value)		(MalemDictionary *dictionary,
						 const gchar     *key,
						 const GValue    *value);
	void		  (* remove_key)	(MalemDictionary *dictionary,
						 const gchar     *key);
	gboolean	  (* has_key)		(MalemDictionary *dictionary,
						 const gchar     *key);
	gchar		**(* list_keys)		(MalemDictionary *dictionary);

	/* Signals */
	void		  (* value_added)	(MalemDictionary *dictionary,
						 const gchar	 *key);
	void		  (* value_changed)	(MalemDictionary *dictionary,
						 const gchar     *key);
	void		  (* key_removed)	(MalemDictionary *dictionary,
						 const gchar     *key);
};

GType   	  malem_dictionary_get_type	(void) G_GNUC_CONST;

gboolean	  malem_dictionary_get_value	(MalemDictionary *dictionary,
						 const gchar     *key,
						 GValue          *value);
void		  malem_dictionary_set_value	(MalemDictionary *dictionary,
						 const gchar     *key,
						 const GValue    *value);
void		  malem_dictionary_remove_key	(MalemDictionary *dictionary,
						 const gchar     *key);
gboolean	  malem_dictionary_has_key	(MalemDictionary *dictionary,
						 const gchar     *key);
gchar		**malem_dictionary_list_keys	(MalemDictionary *dictionary);

void		  malem_dictionary_value_added	(MalemDictionary *dictionary,
						 const gchar     *key);
void		  malem_dictionary_value_changed
						(MalemDictionary *dictionary,
						 const gchar     *key);
void		  malem_dictionary_key_removed	(MalemDictionary *dictionary,
						 const gchar     *key);

/* This inits the interface with the default implementation for task */
void 		  malem_dictionary_iface_init_default	(MalemDictionaryIface *iface);
void 		  malem_dictionary_init_default		(MalemDictionary      *dictionary);

G_END_DECLS

#endif
