/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __MALEM_FACTORY_H__
#define __MALEM_FACTORY_H__

#include <glib.h>
#include <glib-object.h>
#include "malem-task.h"

G_BEGIN_DECLS

#define MALEM_TYPE_FACTORY		(malem_factory_get_type ())
#define MALEM_FACTORY(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), MALEM_TYPE_FACTORY, MalemFactory))
#define MALEM_FACTORY_IFACE(obj)	(G_TYPE_CHECK_CLASS_CAST ((obj), MALEM_TYPE_FACTORY, MalemFactoryIface))
#define MALEM_IS_FACTORY(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), MALEM_TYPE_FACTORY))
#define MALEM_FACTORY_GET_IFACE(obj)	(G_TYPE_INSTANCE_GET_INTERFACE ((obj), MALEM_TYPE_FACTORY, MalemFactoryIface))

typedef struct _MalemFactory		MalemFactory; /* dummy */
typedef struct _MalemFactoryIface	MalemFactoryIface;

struct _MalemFactoryIface {
	GTypeInterface interface;
	
	/* VTable */
	MalemTask		*(* register_task)	(MalemFactory  *factory,
							 const gchar  **interfaces,
							 const gchar   *title,
							 const gchar   *owner);
	MalemTask		**(* list_tasks)	(MalemFactory  *factory);
	MalemTask		**(* list_tasks_for_interface)
							(MalemFactory  *factory,
							const gchar    *inerface);
	void			 (* get_version)	(MalemFactory  *factory,
							 gint          *major,
							 gint          *minor,
							 gint          *micro);
	/* Signals */
	void			 (* task_registered)	(MalemFactory  *factory,
							 GObject       *task,
							 const gchar  **interfaces);
};

GType			 malem_factory_get_type		(void) G_GNUC_CONST;

MalemTask		*malem_factory_register_task	(MalemFactory  *factory,
							 const gchar  **interfaces,
							 const gchar   *title,
							 const gchar   *owner);
MalemTask		**malem_factory_list_tasks	(MalemFactory  *factory);
MalemTask		**malem_factory_list_tasks_for_interface
							(MalemFactory  *factory,
							 const gchar   *interfaces);

void			 malem_factory_get_version	(MalemFactory  *factory,
							 gint          *major,
							 gint          *minor,
							 gint          *micro);

void			 malem_factory_task_registered  (MalemFactory  *factory,
							 MalemTask     *task,
							 const gchar  **interfaces);

G_END_DECLS

#endif
