/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __MALEM_FILE_TRANSFER_H__
#define __MALEM_FILE_TRANSFER_H__

#include <glib-object.h>
#include "malem-enum-types.h"

G_BEGIN_DECLS

typedef enum {
	MALEM_FILE_TRANSFER_DIRECTION_NONE = 0,
	MALEM_FILE_TRANSFER_DIRECTION_FROM = 1,
	MALEM_FILE_TRANSFER_DIRECTION_TO   = 2,
	MALEM_FILE_TRANSFER_DIRECTION_BOTH = 3,
} MalemFileTransferDirection;

#define MALEM_TYPE_FILE_TRANSFER		(malem_file_transfer_get_type ())
#define MALEM_FILE_TRANSFER(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), MALEM_TYPE_FILE_TRANSFER, MalemFileTransfer))
#define MALEM_FILE_TRANSFER_CLASS(obj)		(G_TYPE_CHECK_CLASS_CAST ((obj), MALEM_TYPE_FILE_TRANSFER, MalemFileTransferIface))
#define MALEM_IS_FILE_TRANSFER(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), MALEM_TYPE_FILE_TRANSFER))
#define MALEM_FILE_TRANSFER_GET_IFACE(obj)	(G_TYPE_INSTANCE_GET_INTERFACE ((obj), MALEM_TYPE_FILE_TRANSFER, MalemFileTransferIface))

typedef struct _MalemFileTransfer	MalemFileTransfer; /* dummy */
typedef struct _MalemFileTransferIface	MalemFileTransferIface;

struct _MalemFileTransferIface {
	GTypeInterface interface;

	/* virtual table */
	void	  (* add_file)			(MalemFileTransfer          *transfer,
						 const gchar                *filename,
						 MalemFileTransferDirection  direction);
	MalemFileTransferDirection
		  (* get_file_direction)	(MalemFileTransfer          *transfer,
		 				 const gchar                *filename);
	gchar 	**(* get_files_for_directory)	(MalemFileTransfer          *transfer,
						 const gchar                *directory);

	/* Signals */
	void	  (* file_added)		(MalemFileTransfer          *transfer,
						 const gchar                *filename);
};

GType     malem_file_transfer_get_type			(void) G_GNUC_CONST;

void	  malem_file_transfer_add_file			(MalemFileTransfer          *transfer,
							 const gchar                *filename,
							 MalemFileTransferDirection  direction);
MalemFileTransferDirection
	  malem_file_transfer_get_file_direction	(MalemFileTransfer          *transfer,
		 					 const gchar                *filename);
gchar 	**malem_file_transfer_get_files_for_directory	(MalemFileTransfer          *transfer,
							 const gchar                *directory);

void	  malem_file_transfer_file_added		(MalemFileTransfer          *transfer,
							 const gchar                *filename);

/* This inits the interface with the default implementation for task */
void 	 malem_file_transfer_iface_init_default	(MalemFileTransferIface *iface);
void 	 malem_file_transfer_init_default	(MalemFileTransfer      *transfer);

G_END_DECLS

#endif
