/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <glib.h>
#include <dbus/dbus-glib.h>
#include "malem-frobnicator.h"
#include "client/malem-task-client.h"

#define PROXY_DATA_KEY "malem-frobnicator-dbus-proxy"

static DBusGProxy *
get_proxy (MalemFrobnicator *frobnicator)
{
	DBusGProxy *proxy;
	MalemTaskClient *client = MALEM_TASK_CLIENT (frobnicator);

	/* return null if task terminated */
	if (client->proxy == NULL)
		return NULL;
	
	proxy = DBUS_G_PROXY (g_object_get_data (G_OBJECT (frobnicator),
						 PROXY_DATA_KEY));

	if (proxy == NULL) {
		DBusGConnection *connection;
	        GError *error = NULL;
	
		connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);

		if (connection == NULL) {
			g_printerr ("Failed to open connection to bus: %s\n",
        	        	    error->message);
			g_error_free (error);
			return NULL;
		}
	
		proxy = dbus_g_proxy_new_for_name (connection,
						   "org.gnome.Mathusalem",
						   client->object_path,
						   "org.gnome.Mathusalem.Frobnicator");

		g_object_set_data_full (G_OBJECT (frobnicator),
					PROXY_DATA_KEY,
					(gpointer) proxy,
					(GDestroyNotify) g_object_unref);
	}

	return proxy;
}

static gchar *
impl_say_hello (MalemFrobnicator *frobnicator)
{
	DBusGProxy *proxy = get_proxy(frobnicator);
	gchar *answer;
	GError *error = NULL;
	
	if (proxy == NULL) {
		return NULL;
	} else if (!dbus_g_proxy_call (proxy, "SayHello", 
				       &error,
				       G_TYPE_INVALID,
				       G_TYPE_STRING, &answer,
				       G_TYPE_INVALID)) {
		g_printerr ("Error while saying hello: %s\n", 
			    error->message);
		g_error_free (error);
		return NULL;
	} else {
		return answer;
	}
}

void
malem_frobnicator_init_default (MalemFrobnicator *frobnicator)
{
	get_proxy (frobnicator);
}

void
malem_frobnicator_iface_init_default (MalemFrobnicatorIface *iface)
{
	iface->say_hello = impl_say_hello;
}
