/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __MALEM_TASK_H__
#define __MALEM_TASK_H__

#include <glib.h>
#include <glib-object.h>
#include "malem-enum-types.h"

G_BEGIN_DECLS

#define MALEM_TYPE_TASK			(malem_task_get_type ())
#define MALEM_TASK(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), MALEM_TYPE_TASK, MalemTask))
#define MALEM_TASK_IFACE(obj)		(G_TYPE_CHECK_CLASS_CAST ((obj), MALEM_TYPE_TASK, MalemTaskIface))
#define MALEM_IS_TASK(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), MALEM_TYPE_TASK))
#define MALEM_TASK_GET_IFACE(obj)	(G_TYPE_INSTANCE_GET_INTERFACE ((obj), MALEM_TYPE_TASK, MalemTaskIface))

typedef struct _MalemTask	MalemTask; /* dummy */
typedef struct _MalemTaskIface	MalemTaskIface;

typedef enum {
	MALEM_TASK_STATUS_NEW,
	MALEM_TASK_STATUS_STARTED,
	MALEM_TASK_STATUS_PAUSED,
	MALEM_TASK_STATUS_ABORTED,
	MALEM_TASK_STATUS_COMPLETED
} MalemTaskStatus;

struct _MalemTaskIface {
	GTypeInterface interface;

	/* VTable */
	void		  (* start)			(MalemTask   *task);
	void		  (* pause)			(MalemTask   *task);
	void		  (* abort)			(MalemTask   *task);
	void		  (* add_action)		(MalemTask   *task,
							 const gchar *action_name,
							 const gchar *label,
							 const gchar *icon_name);
	gboolean	  (* has_action)		(MalemTask   *task,
							 const gchar *action_name);
	gchar		**(* list_actions)		(MalemTask   *task);
	void		  (* trigger_action)		(MalemTask   *task,
							 const gchar *action_name);
	gboolean	  (* implements_interface)	(MalemTask   *task,
							 GType        iface);
	GType		 *(* list_interfaces)		(MalemTask   *task);
	/* Note: s/void/gboolean/ to give feedback about succes/failure ? */
	
	/* Signals */
	void		  (* progress_changed)		(MalemTask   *task,
							 guint        progress);
	void		  (* status_changed)		(MalemTask       *task,
							 MalemTaskStatus  status);
	void		  (* action_triggered)		(MalemTask   *task,
							 const gchar *action_name);
};

GType		 malem_task_get_type			(void) G_GNUC_CONST;

/* Status */
MalemTaskStatus	  malem_task_get_status			(MalemTask   *task);
void		  malem_task_start			(MalemTask   *task);
void		  malem_task_pause			(MalemTask   *task);
void		  malem_task_abort			(MalemTask   *task);

void		  malem_task_status_changed		(MalemTask       *task,
							 MalemTaskStatus  status);

/* Progression */
void		  malem_task_set_progress		(MalemTask   *task,
							 guint        progress);
guint		  malem_task_get_progress		(MalemTask   *task);

void		  malem_task_progress_changed		(MalemTask   *task,
							 guint        progress);

/* Properties */
gchar		 *malem_task_get_title			(MalemTask   *task);
gchar		 *malem_task_get_owner			(MalemTask   *task);

/* Custom actions support */
void		  malem_task_add_action			(MalemTask   *task,
							 const gchar *action_name,
							 const gchar *label,
							 const gchar *icon_name);
gboolean	  malem_task_has_action			(MalemTask   *task,
							 const gchar *action_name);
gchar		**malem_task_list_actions		(MalemTask   *task);
void		  malem_task_trigger_action		(MalemTask   *task,
							 const gchar *action_name);

void		  malem_task_action_triggered		(MalemTask   *task,
							 const gchar *action_name);

/* Custom interfaces support */
gboolean	  malem_task_implements_interface	(MalemTask   *task,
							 GType        iface);
GType		 *malem_task_list_interfaces		(MalemTask   *task);

G_END_DECLS

#endif
