/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <glib-object.h>
#include "malem-debug.h"

typedef enum {
	MALEM_DEBUG_NONE	= 0,
	MALEM_DEBUG_APP		= 1 << 0,
	MALEM_DEBUG_TASK	= 1 << 1,
	MALEM_DEBUG_TASK_LIST	= 1 << 2,
	MALEM_DEBUG_DBUS_PROXY	= 1 << 3,
	MALEM_DEBUG_TRAY_ICON   = 1 << 4
} MalemDebugFlags;

static const GFlagsValue flags_values[] = {
	{ MALEM_DEBUG_NONE,
	  "MALEM_DEBUG_NONE",
	  "none" },
	{ MALEM_DEBUG_APP,
	  "MALEM_DEBUG_APP",
	  "app" },
	{ MALEM_DEBUG_TASK,
	  "MALEM_DEBUG_TASK",
	  "task" },
	{ MALEM_DEBUG_TASK_LIST,
	  "MALEM_DEBUG_TASK_LIST",
	  "task-list" },
	{ MALEM_DEBUG_DBUS_PROXY,
	  "MALEM_DEBUG_DBUS_PROXY",
	  "dbus-proxy" },
	{ MALEM_DEBUG_TRAY_ICON,
	  "MALEM_DEBUG_TRAY_ICON",
	  "tray-icon" },
	{ 0, NULL, NULL }
};

static MalemDebugFlags debug_flags = MALEM_DEBUG_NONE;

#if 0
#define MALEM_TYPE_DEBUG_FLAGS		(malem_debug_flags_get_type())

static GType
gedit_debug_section_get_type (void) G_GNUC_CONST
{
	static GType the_type = NULL;
	
	if (the_type == NULL) {
		the_type = g_flags_register_static (
				g_intern_static_string ("MalemDebugFlags"),
				flags_values);
	}
	return the_type;
}
#endif

void
malem_debug_init (void)
{
	const GFlagsValue *value;

	if (g_getenv ("MALEM_DEBUG") != NULL) {
		/* enable all debugging */
		debug_flags = ~MALEM_DEBUG_NONE;
		return;
	}

	for (value = flags_values; value->value_name; value++)
		if (g_getenv (value->value_name) != NULL)
			debug_flags |= value->value;
}

static MalemDebugFlags
get_flag_from_nick (const gchar *nick)
{
	const GFlagsValue *value;
	
	/*
	 * The following code is equivalent to these lines (see genum.c):
	 *   value = g_flags_get_value_by_nick (MALEM_TYPE_DEBUG_FLAGS,
	 * 					section);
	 *   g_return_val_if_fail (value != NULL, MALEM_DEBUG_NONE);
	 *   return value->value;
	 */

	g_return_val_if_fail (nick != NULL, MALEM_DEBUG_NONE);

	for (value = flags_values; value->value_name; value++)
		if (value->value_nick && strcmp (nick, value->value_nick) == 0)
			return value->value;

	g_return_val_if_reached (MALEM_DEBUG_NONE);
}

void
malem_debug_real (const gchar *section,
		  const gchar *file,
		  gint         line,
		  const gchar *function,
		  const gchar *format,
		  ...)
{
	MalemDebugFlags flag = get_flag_from_nick (section);
	
	if (!G_UNLIKELY (debug_flags & flag))
		return;

	if (format == NULL) {
		g_print ("%s:%d (%s)\n",
			 file, line, function);
		fflush (stdout);
	} else {
		va_list args;
		gchar *msg;

		va_start (args, format);
		msg = g_strdup_vprintf (format, args);
		va_end (args);

		g_print ("%s:%d (%s) %s\n",
			 file, line, function, msg);
		fflush (stdout);

		g_free (msg);
	}
}
