/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include "malem-task-list-iter.h"
#include "malem-task-list.h"

static gboolean
iter_is_valid (const MalemTaskListIter *iter)
{
	return iter != NULL && 
	       iter->llink != NULL &&
	       iter->tasklist != NULL &&
	       iter->stamp == MALEM_TASK_LIST (iter->tasklist)->stamp;
}

MalemTaskListIter *
malem_task_list_iter_copy (const MalemTaskListIter *iter)
{
	MalemTaskListIter *new_iter;
	
	g_return_val_if_fail (iter != NULL, NULL);
	
	new_iter = g_slice_new (MalemTaskListIter);
	*new_iter = *iter;
	return new_iter;
}

void
malem_task_list_iter_free (MalemTaskListIter *iter)
{
	g_return_if_fail (iter != NULL);
	g_slice_free (MalemTaskListIter, iter);
}

GType
malem_task_list_iter_get_type (void)
{
	static GType the_type = 0;

	if (G_UNLIKELY (the_type == 0))
		the_type = g_boxed_type_register_static ("MalemTaskListIter",
							 (GBoxedCopyFunc) malem_task_list_iter_copy,
							 (GBoxedFreeFunc) malem_task_list_iter_free);
	return the_type;
}

guint
malem_task_list_iter_get_position (MalemTaskListIter *iter)
{
	g_return_val_if_fail (iter_is_valid (iter), 0);

	return g_list_position (MALEM_TASK_LIST (iter->tasklist)->first,
				iter->llink);
}

MalemTask *
malem_task_list_iter_get_task (MalemTaskListIter *iter)
{
	g_return_val_if_fail (iter_is_valid (iter), NULL);

	return MALEM_TASK (iter->llink->data);
}

gboolean
malem_task_list_iter_previous (MalemTaskListIter *iter)
{
	g_return_val_if_fail (iter_is_valid (iter), FALSE);
	
	if (iter->llink->prev == NULL)
		return FALSE;

	iter->llink = iter->llink->prev;
	return TRUE;
}

gboolean
malem_task_list_iter_next (MalemTaskListIter *iter)
{
	g_return_val_if_fail (iter_is_valid (iter), FALSE);
	
	if (iter->llink->next == NULL)
		return FALSE;

	iter->llink = iter->llink->next;
	return TRUE;
}

gboolean
malem_task_list_iter_is_first (MalemTaskListIter *iter)
{
	g_return_val_if_fail (iter_is_valid (iter), FALSE);
	
	return iter->llink->prev == NULL;
}

gboolean
malem_task_list_iter_is_last (MalemTaskListIter *iter)
{
	g_return_val_if_fail (iter_is_valid (iter), FALSE);
	
	return iter->llink->next == NULL;
}
