/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Steve Frécinaux <steve@istique.net>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __MALEM_TASK_LIST_H__
#define __MALEM_TASK_LIST_H__

#include <glib.h>
#include <glib-object.h>
#include "interfaces/malem-task.h"
#include "malem-task-list-iter.h"

G_BEGIN_DECLS

#define MALEM_TYPE_TASK_LIST		(malem_task_list_get_type ())
#define MALEM_TASK_LIST(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), MALEM_TYPE_TASK_LIST, MalemTaskList))
#define MALEM_TASK_LIST_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k), MALEM_TYPE_TASK_LIST, MalemTaskListClass))
#define MALEM_IS_TASK_LIST(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), MALEM_TYPE_TASK_LIST))
#define MALEM_IS_TASK_LIST_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), MALEM_TYPE_TASK_LIST))
#define MALEM_TASK_LIST_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), MALEM_TYPE_TASK_LIST, MalemTaskListClass))

typedef struct _MalemTaskListPrivate MalemTaskListPrivate;

typedef struct _MalemTaskList {
	GObject               object;
	
	/*< private >*/
	guint                 stamp;
	GList                *first;
	GList                *last;
	guint                 length;
} MalemTaskList;

typedef struct _MalemTaskListClass {
	GObjectClass object_class;
	
	void	(* task_added)		(MalemTaskList     *tasklist,
					 MalemTaskListIter *iter);
	void	(* task_removed)	(MalemTaskList     *tasklist,
					 MalemTaskListIter *task);
} MalemTaskListClass;

GType		 malem_task_list_get_type	(void);
MalemTaskList	*malem_task_list_new		(void);
void		 malem_task_list_add_task	(MalemTaskList     *tasklist,
						 MalemTask         *task);
gboolean	 malem_task_list_remove		(MalemTaskList     *tasklist,
						 MalemTaskListIter *iter);
void		 malem_task_list_find		(MalemTaskList     *tasklist,
						 MalemTaskListIter *iter,
						 MalemTask         *task);

guint		 malem_task_list_get_length	(MalemTaskList *tasklist);

void		 malem_task_list_get_iter_first		(MalemTaskList     *tasklist,
							 MalemTaskListIter *iter);
void		 malem_task_list_get_iter_last		(MalemTaskList     *tasklist,
							 MalemTaskListIter *iter);
void		 malem_task_list_get_iter_at_position	(MalemTaskList     *tasklist,
							 MalemTaskListIter *iter,
							 guint              pos);

G_END_DECLS

#endif
